@@ -16,6 +16,8 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
+#include <config.h>
+
 #include <sys/time.h>
 #include <sys/types.h>
 
@@ -28,7 +30,6 @@
 
 #include <dev/usb/usb.h>
 
-#include "libusb.h"
 #include "libusbi.h"
 
 struct device_priv {
@@ -40,7 +41,6 @@ struct device_priv {
 };
 
 struct handle_priv {
-	int pipe[2];				/* for event notification */
 	int endpoints[USB_MAX_ENDPOINTS];
 };
 
@@ -74,8 +74,7 @@ static void netbsd_destroy_device(struct libusb_device *);
 static int netbsd_submit_transfer(struct usbi_transfer *);
 static int netbsd_cancel_transfer(struct usbi_transfer *);
 static void netbsd_clear_transfer_priv(struct usbi_transfer *);
-static int netbsd_handle_events(struct libusb_context *ctx, struct pollfd *,
-    nfds_t, int);
+static int netbsd_handle_transfer_completion(struct usbi_transfer *);
 static int netbsd_clock_gettime(int, struct timespec *);
 
 /*
@@ -87,11 +86,12 @@ static int _sync_control_transfer(struct usbi_transfer *);
 static int _sync_gen_transfer(struct usbi_transfer *);
 static int _access_endpoint(struct libusb_transfer *);
 
-const struct usbi_os_backend netbsd_backend = {
+const struct usbi_os_backend usbi_backend = {
 	"Synchronous NetBSD backend",
 	0,
 	NULL,				/* init() */
 	NULL,				/* exit() */
+	NULL,				/* set_option() */
 	netbsd_get_device_list,
 	NULL,				/* hotplug_poll */
 	netbsd_open,
@@ -115,6 +115,9 @@ const struct usbi_os_backend netbsd_backend = {
 	NULL,				/* alloc_streams */
 	NULL,				/* free_streams */
 
+	NULL,				/* dev_mem_alloc() */
+	NULL,				/* dev_mem_free() */
+
 	NULL,				/* kernel_driver_active() */
 	NULL,				/* detach_kernel_driver() */
 	NULL,				/* attach_kernel_driver() */
@@ -125,13 +128,14 @@ const struct usbi_os_backend netbsd_backend = {
 	netbsd_cancel_transfer,
 	netbsd_clear_transfer_priv,
 
-	netbsd_handle_events,
+	NULL,				/* handle_events() */
+	netbsd_handle_transfer_completion,
 
 	netbsd_clock_gettime,
+	0,				/* context_priv_size */
 	sizeof(struct device_priv),
 	sizeof(struct handle_priv),
 	0,				/* transfer_priv_size */
-	0,				/* add_iso_packet_size */
 };
 
 int
@@ -210,8 +214,9 @@ error:
 int
 netbsd_open(struct libusb_device_handle *handle)
 {
-	struct handle_priv *hpriv = (struct handle_priv *)handle->os_priv;
 	struct device_priv *dpriv = (struct device_priv *)handle->dev->os_priv;
+	struct handle_priv *hpriv = (struct handle_priv *)handle->os_priv;
+	int i;
 
 	dpriv->fd = open(dpriv->devnode, O_RDWR);
 	if (dpriv->fd < 0) {
@@ -220,29 +225,23 @@ netbsd_open(struct libusb_device_handle *handle)
 			return _errno_to_libusb(errno);
 	}
 
+	for (i = 0; i < USB_MAX_ENDPOINTS; i++)
+		hpriv->endpoints[i] = -1;
+
 	usbi_dbg("open %s: fd %d", dpriv->devnode, dpriv->fd);
 
-	if (pipe(hpriv->pipe) < 0)
-		return _errno_to_libusb(errno);
-
-	return usbi_add_pollfd(HANDLE_CTX(handle), hpriv->pipe[0], POLLIN);
+	return (LIBUSB_SUCCESS);
 }
 
 void
 netbsd_close(struct libusb_device_handle *handle)
 {
-	struct handle_priv *hpriv = (struct handle_priv *)handle->os_priv;
 	struct device_priv *dpriv = (struct device_priv *)handle->dev->os_priv;
 
 	usbi_dbg("close: fd %d", dpriv->fd);
 
 	close(dpriv->fd);
 	dpriv->fd = -1;
-
-	usbi_remove_pollfd(HANDLE_CTX(handle), hpriv->pipe[0]);
-
-	close(hpriv->pipe[0]);
-	close(hpriv->pipe[1]);
 }
 
 int
@@ -469,8 +468,7 @@ netbsd_submit_transfer(struct usbi_transfer *itransfer)
 	if (err)
 		return (err);
 
-	if (write(hpriv->pipe[1], &itransfer, sizeof(itransfer)) < 0)
-		return _errno_to_libusb(errno);
+	usbi_signal_transfer_completion(itransfer);
 
 	return (LIBUSB_SUCCESS);
 }
@@ -492,63 +490,9 @@ netbsd_clear_transfer_priv(struct usbi_transfer *itransfer)
 }
 
 int
-netbsd_handle_events(struct libusb_context *ctx, struct pollfd *fds, nfds_t nfds,
-    int num_ready)
+netbsd_handle_transfer_completion(struct usbi_transfer *itransfer)
 {
-	struct libusb_device_handle *handle;
-	struct handle_priv *hpriv = NULL;
-	struct usbi_transfer *itransfer;
-	struct pollfd *pollfd;
-	int i, err = 0;
-
-	usbi_dbg("");
-
-	pthread_mutex_lock(&ctx->open_devs_lock);
-	for (i = 0; i < nfds && num_ready > 0; i++) {
-		pollfd = &fds[i];
-
-		if (!pollfd->revents)
-			continue;
-
-		hpriv = NULL;
-		num_ready--;
-		list_for_each_entry(handle, &ctx->open_devs, list,
-		    struct libusb_device_handle) {
-			hpriv = (struct handle_priv *)handle->os_priv;
-
-			if (hpriv->pipe[0] == pollfd->fd)
-				break;
-
-			hpriv = NULL;
-		}
-
-		if (NULL == hpriv) {
-			usbi_dbg("fd %d is not an event pipe!", pollfd->fd);
-			err = ENOENT;
-			break;
-		}
-
-		if (pollfd->revents & POLLERR) {
-			usbi_remove_pollfd(HANDLE_CTX(handle), hpriv->pipe[0]);
-			usbi_handle_disconnect(handle);
-			continue;
-		}
-
-		if (read(hpriv->pipe[0], &itransfer, sizeof(itransfer)) < 0) {
-			err = errno;
-			break;
-		}
-
-		if ((err = usbi_handle_transfer_completion(itransfer,
-		    LIBUSB_TRANSFER_COMPLETED)))
-			break;
-	}
-	pthread_mutex_unlock(&ctx->open_devs_lock);
-
-	if (err)
-		return _errno_to_libusb(err);
-
-	return (LIBUSB_SUCCESS);
+	return usbi_handle_transfer_completion(itransfer, LIBUSB_TRANSFER_COMPLETED);
 }
 
 int
